﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking;


namespace HeuristicLab.Encodings.PackingEncoding.MultiComponentVector {
  [Item("MultiComponentVectorMove", "An abstract move for a multi component vector encoded solution.")]
  [StorableClass]
  public abstract class MultiComponentVectorMove : Item, IPackingMove {
    [Storable]
    public MultiComponentVectorEncoding MultiComponentVector { get; protected set; }

    [StorableConstructor]
    protected MultiComponentVectorMove(bool deserializing) : base(deserializing) { }
    protected MultiComponentVectorMove(MultiComponentVectorMove original, Cloner cloner)
      : base(original, cloner) {
        if (original.MultiComponentVector != null)
          this.MultiComponentVector = cloner.Clone(original.MultiComponentVector);
    }
    public MultiComponentVectorMove(MultiComponentVectorEncoding multiComponentVector)
      : base() {
        MultiComponentVector = multiComponentVector;
    }

    public abstract IPackingSolutionEncoding GetSolutionAfterMove();
    public abstract Type GetMoveAttributeType();
    public abstract MultiComponentVectorMoveAttribute GetAttribute(double quality);
    public abstract bool BelongsToAttribute (MultiComponentVectorMoveAttribute attribute, bool hardCriterion);
  }
}